var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';

import { TransitionMotion, spring } from 'react-motion';
import compose from 'recompose/compose';
import defaultProps from 'recompose/defaultProps';
import withPropsOnChange from 'recompose/withPropsOnChange';
import pure from 'recompose/pure';
import { withTheme, withColors, withDimensions, withMotion } from '../../../hocs';
import { getInheritedColorGenerator } from '../../../lib/colorUtils';
import { generateGroupedBars, generateStackedBars } from '../../../lib/charts/bar';
import { getAccessorFor } from '../../../lib/propertiesConverters';
import Container from '../Container';
import SvgWrapper from '../SvgWrapper';
import Grid from '../../axes/Grid';
import CartesianMarkers from '../../cartesian/markers/CartesianMarkers';
import Axes from '../../axes/Axes';

var HeatMap = function HeatMap(_ref) {
    var data = _ref.data,
        getIndex = _ref.getIndex,
        keys = _ref.keys,
        groupMode = _ref.groupMode,
        layout = _ref.layout,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        xPadding = _ref.xPadding,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableLabels = _ref.enableLabels,
        getLabelsLinkColor = _ref.getLabelsLinkColor,
        getLabelsTextColor = _ref.getLabelsTextColor,
        markers = _ref.markers,
        theme = _ref.theme,
        getColor = _ref.getColor,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var columnCount = keys.length;
    var rowCount = data.length;

    var cellWidth = width / columnCount;
    var cellHeight = height / rowCount;

    var cellSize = Math.min(cellWidth, cellHeight);

    var cells = data.reduce(function (acc, d, i) {
        keys.forEach(function (key, j) {
            acc.push({
                key: key + '.' + getIndex(d),
                xKey: key,
                yKey: getIndex(d),
                x: j * cellSize + cellSize * 0.5,
                y: i * cellSize + cellSize * 0.5,
                value: d[key]
            });
        });

        return acc;
    }, []);

    /*
    console.log(data)
    console.log(keys)
    console.log('columns:', columnCount, 'rows', rowCount)
    console.log('cellWidth:', cellWidth, 'cellHeight:', cellHeight, 'cellSize:', cellSize)
    console.log('cells:', cells)
    */

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;

            return React.createElement(
                SvgWrapper,
                { width: outerWidth, height: outerHeight, margin: margin },
                cells.map(function (cell) {
                    return React.createElement(
                        'g',
                        { key: cell.key, transform: 'translate(' + cell.x + ', ' + cell.y + ')' },
                        React.createElement('rect', {
                            fill: '#F00',
                            fillOpacity: 0.35,
                            stroke: '#900',
                            strokeWidth: 1,
                            x: cellSize * -0.5,
                            y: cellSize * -0.5,
                            width: cellSize,
                            height: cellSize
                        }),
                        React.createElement(
                            'text',
                            {
                                alignmentBaseline: 'central',
                                textAnchor: 'middle',
                                fill: '#000'
                            },
                            cell.value
                        )
                    );
                })
            );
        }
    );

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var result = void 0;
    if (groupMode === 'grouped') {
        result = generateGroupedBars(layout, data, getIndex, keys, width, height, getColor, {
            xPadding: xPadding
        });
    } else if (groupMode === 'stacked') {
        result = generateStackedBars(layout, data, getIndex, keys, width, height, getColor, {
            xPadding: xPadding
        });
    }

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;

            var bars = void 0;
            if (animate === true) {
                bars = React.createElement(
                    TransitionMotion,
                    {
                        styles: result.bars.map(function (bar) {
                            return {
                                key: bar.key,
                                data: bar,
                                style: {
                                    x: spring(bar.x, motionProps),
                                    y: spring(bar.y, motionProps),
                                    width: spring(bar.width, motionProps),
                                    height: spring(bar.height, motionProps)
                                }
                            };
                        })
                    },
                    function (interpolatedStyles) {
                        return React.createElement(
                            'g',
                            null,
                            interpolatedStyles.map(function (_ref4) {
                                var key = _ref4.key,
                                    style = _ref4.style,
                                    data = _ref4.data;
                                return React.createElement(BarItem, _extends({
                                    key: key
                                }, data, style, {
                                    showTooltip: showTooltip,
                                    hideTooltip: hideTooltip,
                                    theme: theme
                                }));
                            })
                        );
                    }
                );
            } else {
                bars = result.bars.map(function (d) {
                    return React.createElement(BarItem, _extends({
                        key: d.key
                    }, d, {
                        showTooltip: showTooltip,
                        hideTooltip: hideTooltip,
                        theme: theme
                    }));
                });
            }

            return React.createElement(
                SvgWrapper,
                { width: outerWidth, height: outerHeight, margin: margin },
                React.createElement(Grid, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? result.xScale : null,
                    yScale: enableGridY ? result.yScale : null
                }, motionProps)),
                React.createElement(CartesianMarkers, {
                    markers: markers,
                    width: width,
                    height: height,
                    xScale: result.xScale,
                    yScale: result.yScale,
                    theme: theme
                }),
                React.createElement(Axes, _extends({
                    xScale: result.xScale,
                    yScale: result.yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                bars,
                enableLabels && result.bars.map(function (d) {
                    return React.createElement(BarItemLabel, _extends({}, d, {
                        textColor: getLabelsTextColor(d, theme),
                        linkColor: getLabelsLinkColor(d, theme)
                    }));
                })
            );
        }
    );
};

HeatMap.propTypes = {
    // data
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    indexBy: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    getIndex: PropTypes.func.isRequired, // computed
    keys: PropTypes.arrayOf(PropTypes.string).isRequired,

    forceSquare: PropTypes.bool.isRequired,

    // axes & grid
    axisTop: PropTypes.object,
    axisRight: PropTypes.object,
    axisBottom: PropTypes.object,
    axisLeft: PropTypes.object,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // labels
    enableLabels: PropTypes.bool.isRequired,
    labelsTextColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    getLabelsTextColor: PropTypes.func.isRequired, // computed

    // theming
    getColor: PropTypes.func.isRequired,

    // interactivity
    isInteractive: PropTypes.bool
};

export var HeatMapDefaultProps = {
    indexBy: 'id',
    keys: ['value'],

    forceSquare: false,

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: true,

    // labels
    enableLabels: true,
    labelsTextColor: 'theme',

    // interactivity
    isInteractive: true
};

HeatMap.defaultProps = HeatMapDefaultProps;

var enhance = compose(defaultProps(HeatMapDefaultProps), withTheme(), withColors(), withDimensions(), withMotion(), withPropsOnChange(['indexBy'], function (_ref5) {
    var indexBy = _ref5.indexBy;
    return {
        getIndex: getAccessorFor(indexBy)
    };
}), withPropsOnChange(['labelsTextColor'], function (_ref6) {
    var labelsTextColor = _ref6.labelsTextColor;
    return {
        getLabelsTextColor: getInheritedColorGenerator(labelsTextColor, 'axis.textColor')
    };
}), pure);

export default enhance(HeatMap);