'use strict';

exports.__esModule = true;
exports.HeatMapDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactMotion = require('react-motion');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _hocs = require('../../../hocs');

var _colorUtils = require('../../../lib/colorUtils');

var _bar = require('../../../lib/charts/bar');

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _CartesianMarkers = require('../../cartesian/markers/CartesianMarkers');

var _CartesianMarkers2 = _interopRequireDefault(_CartesianMarkers);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var HeatMap = function HeatMap(_ref) {
    var data = _ref.data,
        getIndex = _ref.getIndex,
        keys = _ref.keys,
        groupMode = _ref.groupMode,
        layout = _ref.layout,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        xPadding = _ref.xPadding,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableLabels = _ref.enableLabels,
        getLabelsLinkColor = _ref.getLabelsLinkColor,
        getLabelsTextColor = _ref.getLabelsTextColor,
        markers = _ref.markers,
        theme = _ref.theme,
        getColor = _ref.getColor,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var columnCount = keys.length;
    var rowCount = data.length;

    var cellWidth = width / columnCount;
    var cellHeight = height / rowCount;

    var cellSize = Math.min(cellWidth, cellHeight);

    var cells = data.reduce(function (acc, d, i) {
        keys.forEach(function (key, j) {
            acc.push({
                key: key + '.' + getIndex(d),
                xKey: key,
                yKey: getIndex(d),
                x: j * cellSize + cellSize * 0.5,
                y: i * cellSize + cellSize * 0.5,
                value: d[key]
            });
        });

        return acc;
    }, []);

    /*
    console.log(data)
    console.log(keys)
    console.log('columns:', columnCount, 'rows', rowCount)
    console.log('cellWidth:', cellWidth, 'cellHeight:', cellHeight, 'cellSize:', cellSize)
    console.log('cells:', cells)
    */

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive, theme: theme },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;

            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: outerWidth, height: outerHeight, margin: margin },
                cells.map(function (cell) {
                    return _react2.default.createElement(
                        'g',
                        { key: cell.key, transform: 'translate(' + cell.x + ', ' + cell.y + ')' },
                        _react2.default.createElement('rect', {
                            fill: '#F00',
                            fillOpacity: 0.35,
                            stroke: '#900',
                            strokeWidth: 1,
                            x: cellSize * -0.5,
                            y: cellSize * -0.5,
                            width: cellSize,
                            height: cellSize
                        }),
                        _react2.default.createElement(
                            'text',
                            {
                                alignmentBaseline: 'central',
                                textAnchor: 'middle',
                                fill: '#000'
                            },
                            cell.value
                        )
                    );
                })
            );
        }
    );

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var result = void 0;
    if (groupMode === 'grouped') {
        result = (0, _bar.generateGroupedBars)(layout, data, getIndex, keys, width, height, getColor, {
            xPadding: xPadding
        });
    } else if (groupMode === 'stacked') {
        result = (0, _bar.generateStackedBars)(layout, data, getIndex, keys, width, height, getColor, {
            xPadding: xPadding
        });
    }

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive, theme: theme },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;

            var bars = void 0;
            if (animate === true) {
                bars = _react2.default.createElement(
                    _reactMotion.TransitionMotion,
                    {
                        styles: result.bars.map(function (bar) {
                            return {
                                key: bar.key,
                                data: bar,
                                style: {
                                    x: (0, _reactMotion.spring)(bar.x, motionProps),
                                    y: (0, _reactMotion.spring)(bar.y, motionProps),
                                    width: (0, _reactMotion.spring)(bar.width, motionProps),
                                    height: (0, _reactMotion.spring)(bar.height, motionProps)
                                }
                            };
                        })
                    },
                    function (interpolatedStyles) {
                        return _react2.default.createElement(
                            'g',
                            null,
                            interpolatedStyles.map(function (_ref4) {
                                var key = _ref4.key,
                                    style = _ref4.style,
                                    data = _ref4.data;
                                return _react2.default.createElement(BarItem, _extends({
                                    key: key
                                }, data, style, {
                                    showTooltip: showTooltip,
                                    hideTooltip: hideTooltip,
                                    theme: theme
                                }));
                            })
                        );
                    }
                );
            } else {
                bars = result.bars.map(function (d) {
                    return _react2.default.createElement(BarItem, _extends({
                        key: d.key
                    }, d, {
                        showTooltip: showTooltip,
                        hideTooltip: hideTooltip,
                        theme: theme
                    }));
                });
            }

            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: outerWidth, height: outerHeight, margin: margin },
                _react2.default.createElement(_Grid2.default, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? result.xScale : null,
                    yScale: enableGridY ? result.yScale : null
                }, motionProps)),
                _react2.default.createElement(_CartesianMarkers2.default, {
                    markers: markers,
                    width: width,
                    height: height,
                    xScale: result.xScale,
                    yScale: result.yScale,
                    theme: theme
                }),
                _react2.default.createElement(_Axes2.default, _extends({
                    xScale: result.xScale,
                    yScale: result.yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                bars,
                enableLabels && result.bars.map(function (d) {
                    return _react2.default.createElement(BarItemLabel, _extends({}, d, {
                        textColor: getLabelsTextColor(d, theme),
                        linkColor: getLabelsLinkColor(d, theme)
                    }));
                })
            );
        }
    );
};

HeatMap.propTypes = {
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
    indexBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getIndex: _propTypes2.default.func.isRequired, // computed
    keys: _propTypes2.default.arrayOf(_propTypes2.default.string).isRequired,

    forceSquare: _propTypes2.default.bool.isRequired,

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // labels
    enableLabels: _propTypes2.default.bool.isRequired,
    labelsTextColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getLabelsTextColor: _propTypes2.default.func.isRequired, // computed

    // theming
    getColor: _propTypes2.default.func.isRequired,

    // interactivity
    isInteractive: _propTypes2.default.bool
};

var HeatMapDefaultProps = exports.HeatMapDefaultProps = {
    indexBy: 'id',
    keys: ['value'],

    forceSquare: false,

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: true,

    // labels
    enableLabels: true,
    labelsTextColor: 'theme',

    // interactivity
    isInteractive: true
};

HeatMap.defaultProps = HeatMapDefaultProps;

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(HeatMapDefaultProps), (0, _hocs.withTheme)(), (0, _hocs.withColors)(), (0, _hocs.withDimensions)(), (0, _hocs.withMotion)(), (0, _withPropsOnChange2.default)(['indexBy'], function (_ref5) {
    var indexBy = _ref5.indexBy;
    return {
        getIndex: (0, _propertiesConverters.getAccessorFor)(indexBy)
    };
}), (0, _withPropsOnChange2.default)(['labelsTextColor'], function (_ref6) {
    var labelsTextColor = _ref6.labelsTextColor;
    return {
        getLabelsTextColor: (0, _colorUtils.getInheritedColorGenerator)(labelsTextColor, 'axis.textColor')
    };
}), _pure2.default);

exports.default = enhance(HeatMap);